/*====================================================================
  cfgfile.c -> Encapsulates the config and parse file stuff.
  revision $Revision: 2.19 $

             (c) 2000 Santiago Romero AkA NoP / Compiler
                         sromero@unix-shells.com

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ====================================================================*/

#include "cfgfile.h"

#define _(string) gettext(string)

static const char rcsid[]="$Id: cfgfile.c,v 2.19 2004/09/09 11:53:29 alberto Exp $";

/*--------------------------------------------------------------------
   int CreateCFGFile( char *fname );

   This function creates (with user's permission) a basic config file.
--------------------------------------------------------------------*/
int CreateCFGFile( char *fname )
{
   char ans;
   int  numservers, i;
   FILE *fp;
   char server[256], user[256], password[256];
   char port[10];
	char *homedir; /* user's home dir */
	char cfgdir[256]; /* dir where popmail related files go */
	struct stat	statinfo; /* info on config file */
	

   /* Just ask user and create config file: */
   printf(_("> Failed to open config file %s\n"
				"  This means that you haven't still configured this program or\n"
				"  maybe you want to create a new config file (erasing the old \n"
				"  one).\n\n"
				"  Do you want me to create a basic file for you now?\n"
				"  I'll ask you whatever I need to create the file...  (y/n) "), fname);

   ans = GetYesNo();
   if( ans == 0 )
   {
       printf(_("\n  Ok: read "));
		 printf(DESTDOCDIR);
		 printf(_(" and create it by yourself.\n\n"));
       return(0);
   }

	/* set default directory for popmail related files */
	homedir = getenv("HOME");
	sprintf(cfgdir,"%s/.popmail",homedir);

	if (stat (cfgdir, &statinfo) != 0) /* config dir does not exist */
		mkdir (cfgdir, 0700);
	
   fp = fopen( fname, "wt" );
   if ( fp == NULL )
      error(_("Couldn't create config file!\n"));

   fprintf( fp, "#-------------------------------------------------------------------\n");
   fprintf( fp, "#  Generic .popmailrc file for your HOME directory.\n");
   fprintf( fp, "#  By Santiago Romero, santiago.romero@iname.com\n");
   fprintf( fp, "#-------------------------------------------------------------------\n");
   fprintf( fp, "\n");

   fprintf( fp, "#--------------------------------------------------------------------\n");
   fprintf( fp, "# Server info. Put here the number of servers and their info.\n");
   fprintf( fp, "#--------------------------------------------------------------------\n");
   fprintf( fp, "# How many accounts are defined in the file\n");

   printf(_("\n> How many different email accounts do you want to define? (1-n) "));
   scanf("%d", &numservers );
   fgetc(stdin);
   if( numservers <= 0 ) numservers = 1;

   fprintf( fp, "popservers %d\n", numservers);

   for( i=0; i<numservers; i++ )
   {
      fprintf( fp, "\n");
      fprintf(fp, "#----- Account number %d---------------------\n", i+1);

      printf(_("  Account # %d\n"), i+1 );
      printf(_("   -> Mail server name? (ex: pop.server.com): "));
      fgets( server, 255, stdin );
      printf(_("   -> User server name? (ex: milogin): "));
      fgets( user, 255, stdin );
		get_password (password);

      printf(_("   -> POP3 Server port? (ENTER to default, 110): "));
		fgets (port, 10, stdin);

		if (port[0] == '\n') /* set default value */
			strcpy (port, "110\n");
		else if (!isdigit(port[0])) {/* bad entry */
			printf (_("\nBad entry, port should be a number."
						" I will use 110 by default\n"));
			strcpy (port, "110\n");
		}

      fprintf( fp, "server %s", server);
      fprintf( fp, "user %s", user);
      fprintf( fp, "password %s\n", password);
      fprintf( fp, "port %s", port);
   }

	memset (password, 0, sizeof (password)); /* don't want to be readable */


   fprintf( fp, "\n");
   fprintf( fp, "#----- And so on... -------------------------\n");
   fprintf( fp, "\n");

   fprintf( fp, "#--------------------------------------------------------------------\n");
   fprintf( fp, "# Define here different setup options for popmail\n");
   fprintf( fp, "#--------------------------------------------------------------------\n");
   fprintf( fp, "\n");

   printf(_("\n> Generic options:\n"
				"  -> Max size a message can be in order to be fetched (in bytes).\n"
				"   (Enter 0 or press ENTER key to disable this option and fetch\n"
				"   all messages) : "));
   fgets (port, 10, stdin);
	
	if (port[0] == '\n' || !isdigit(port[0])) /* default, no size filtering */
		strcpy (port, "0\n");

   if ( atoi (port) > 0 )
       fprintf( fp, "maxsize %s", port);
   else
       fprintf( fp, "#maxsize 10000000\n");

   printf(_("\n  -> Enter now the file name where you will write the filter\n"
				"   rules. Enter FULL PATH as /home/youruser/.filterfile, press ENTER\n"
				"   key to set to default value or write 0 to disable email filtering\n"
				"   (default %s/filterfile):\n\t> "),cfgdir);
	
   fgets( server, 255, stdin );
   if( server[0] == '0' && strlen(server) < 3 ) /* filtering disabled */
      fprintf( fp, "#filterfile /home/youruser/.filterfile\n");
   else if ( server[0] == '\n' ) /* default destination */
      fprintf( fp, "filterfile %s/filterfile\n", cfgdir);
	else /* user defined */
      fprintf( fp, "filterfile %s", server);

   printf(_("\n  -> Enter now a file name where I will LOG info about deleted\n"
				"   msgs. Enter FULL PATH as /home/youruser/.popmail.log, press\n"
				"   ENTER key to set to default value or write 0 to disable logging\n"
				"   (default %s/popmail.log):\n\t> "), cfgdir);
	
   fgets( server, 255, stdin );
   if( server[0] == '0' && strlen(server) < 3 ) /* logging disabled */
      fprintf( fp, "#logfile /home/youruser/.popmail.log\n\n");
	else if ( server[0] == '\n' ) /* use default */
		fprintf( fp, "logfile %s/popmail.log\n\n",cfgdir);
   else	/* user defined */
      fprintf( fp, "logfile %s\n", server);

   fclose(fp);
   printf(_("\n> Now I'm going to exec \"chmod 0600 .popmailrc\" to set secure\n"
				"  permissions. This way nobody but you will be able to read your\n"
				"  users/passwords: "));
   chmod(fname, S_IRUSR | S_IWUSR | S_IFREG );
   printf(_("done.\n"));

   printf(_("\n> Done! You can now run popmail. Remember to edit\n"
				"  %s file\n"
				"  if you want to add email accounts or change any option. :-)\n\n"),fname);

	return 0;
}


/*--------------------------------------------------------------------
 int ParseFileForNumServers( FILE * );

 This function searches from the beginning of the file till it
 finds the "popservers num" setup option, and returns it to caller.
--------------------------------------------------------------------*/
int ParseFileForNumServers( FILE *fp )
{
   char done = 0;
   char *where;
   char line[256], word[40];
   int numservers;

   /* while we don't find the "popservers number" string, do: */
   while(!done)
   {
      /* get a line */
      if( fgets( line, 255, fp ) != NULL )
      {
         /* if the start of the line is popservers, get the number after it: */
         if (((where=strstr(line, "popservers")) != NULL) &&
					(line[0] != '#')) /* no comment-line */
         {
             if (GetnWord( where, 2, word, sizeof (word)))
                 numservers = atoi( word );
             else
               error(_("Error parsing .popmailrc file. Delete it and run popmail.\n"));
               done = 1;
         }

      
      }
   if( feof(fp) )
         {
             error(_("It Seems as if you have not \"popservers\" directive on"
							 " your popmailrc file, or perhaps it has blanks around.\n"
							 "\nPlease revise your popmailrc file or delete it and run"
							 " popmail again.\n"));
             done = 1;
         }
   }
   return( numservers );
}



/*--------------------------------------------------------------------
 int ParseFileForServers( FILE *, numservers, servers );

 This function reads all the servers from the config file.
--------------------------------------------------------------------*/
int ParseFileForServers( FILE *fp, int numservers,
                         struct Server **servidores )
{
   char done = 0, anerror = 0;
   char line[256], word[128];
   int server;

   /* default values of the servers: */
   /* for( i=0; i< numservers; i++ )
   {
		server_create (servidores[i]);
	}*/

   /* Now scan the file for server, username, password and port strings. */
   /* And each time we find a "server" string, increment SERVER variable. */
   server = -1;
   while(!done)
   {
      if( fgets( line, 255, fp ) != NULL )
      {
			if ((GetFirstWord (line, 255, word)) &&
					(line[0] != '#')){ /* get keyword in a no-comment line */

				if (strcmp (word, "server") == 0) {
      	      server++;
					if (GetnWord (line, 2, word, sizeof (word))) {
						server_set_name (servidores[server], word);
					}
          	  	else
            	    anerror = 1;
				}
				else if (strcmp (word, "user") == 0) { 
					if (GetnWord (line, 2, word, sizeof (word))) {
						server_set_user (servidores[server], word);
					}
            	else
              	  anerror = 1;
        	 	}
				else if (strcmp (word, "password") == 0) { 
	            if (GetnWord (line, 2, word, sizeof (word))) {
						server_set_pass (servidores[server], word);
					}
      	      else
         	      anerror = 1;
         	}
				else if (strcmp (word, "port") == 0) { 
	   	      if (server == numservers-1) /* parsed all servers */
   	   	      done = 1;

	            if (GetnWord (line, 2, word, sizeof (word))) {
						server_set_port (servidores[server], atoi (word));
					}
            	else
               	anerror = 1;
         	}
			}

      }

      /* there was any parsing error? */
      if( anerror || feof(fp) )
      {
          error(_("Error parsing .popmailrc file. Delete it and run popmail.\n"));
      }
   }
   return( numservers );
}


/*--------------------------------------------------------------------
 int ParseFileForOptions( FILE *, int *, Filters * );

 This function reads the rest of popmail options, including the
 filter file and rules.
--------------------------------------------------------------------*/
int ParseFileForOptions( FILE *fp, char *logfname, int *maxsize,
                         struct Filters *filters, char *filterfile)
{
   char done = 0;
   char line[256], word[256];
   int numservers;

   /* by default, filters disabled */
   filters->num_rules = 0;

   /* while we don't find the "popservers number" string, do: */
   while(!done)
   {
      /* get a line */
      if( fgets( line, 255, fp ) != NULL )
      {
         if ((GetFirstWord (line, 255, word)) && 
				(line[0] != '#')) {
				
            /* look for the maxsize option */
            if ((strcmp(word, "maxsize")) == 0) { 
					if (GetnWord (line, 2, word, sizeof (word)))
						*maxsize = atoi( word );
					else {
						done = 1;
						error(_("Error parsing .popmailrc file. Delete it"
									" and run popmail.\n"));
               }
            }
            /* now we try to find the filterfile option */
            else if ((strcmp (word, "filterfile")) == 0) {
               if (GetnWord (line, 2, word, sizeof (word))) {
						 strcpy (filterfile, word);
                   ParseFilterFile (word, filters);
					} 
					else {
                  done = 1;
                  fprintf (stderr, _("Error parsing %s file. "
								"Please check it and rerun popmail.\n"),
								word);
						exit (-1);
               }
            }
            /* now we try to find the filterfile option */
            else if ((strcmp (word, "logfile")) == 0) {
               if (GetnWord (line, 2, word, sizeof (word)))
                  strcpy( logfname, word );
               else {
                  done = 1;
                  error(_("Error parsing .popmailrc file. Delete it"
									" and run popmail.\n"));
               }
            }

         }
      }
      if( feof(fp) )
          done = 1;
   }
   return( numservers );
}

int get_password (char *password)
{
	char *tmppass; /* temporary password to compare both typed */
	int correct_twice;

	correct_twice = 0;
	while (!correct_twice) {
	   tmppass = getpass (_("   -> Account Password? (you will not see what"
				" you're typing) : "));

		strcpy (password, tmppass);
		memset (tmppass, 0, sizeof (tmppass));

		tmppass = getpass (_("   -> Please enter password again: "));

		if (strcmp (tmppass, password) == 0) {
			correct_twice = 1;
			memset (tmppass, 0, sizeof (tmppass)); /* pass not visible anymore */
		}
		else /* passwords don't match */
			printf (_("\n\tPasswords typed are not the same. "
						"Please retype password.\n"));
	}

	return 0;
}
