/***********************************************************************
 * filter.c:  Filtering stuff Routines for POPMail
 * 	$Revision: 1.10 $
 *
 * 	Copyright (c) 2003, 2004 L. Alberto Gimnez Benito	<algibe@teleline.es>
 * 
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * **********************************************************************/

#include <string.h>

#include "filter.h"
#include "util.h"

#define _(string) gettext(string)

#define MAX_BUFFER 4096

static const char rcsid[]="$Id: filter.c,v 1.10 2004/09/10 19:42:05 alberto Exp $";


/************************* PRIVATE FUNCTIONS **********************/
/* 
* Checks for match between FITER and a line in BUFFER
 * Returns:
 * 		 0 if no match
 * 		 1 if match
 */
static int check_filter(struct Rule *filter, char buffer[])
{
	int i;
	int match = 0; /* filter matched? */
	int num_lines;	/* number of lines in the buffer */
	size_t offset;
	char line[MAX_BUFFER];	/* Line of the buffer */

	offset = 0;
	match = 0;
	num_lines = CountLines(buffer);

	for (i = 0; i < num_lines-1; i++) {
		offset = ExtractLine(buffer, MAX_BUFFER, offset, line);
		match = (regexec(&(filter->rule), line, 0, NULL, 0) == 0);
#ifdef DEBUG
		printf("'%s' --> '%s'... %s\n", line, filter->rule_text,
				match ? " *** match! ***" : " *no match*");
#endif
		if (match)
			break;
	}

	return match;
}

/* msg_parse_from: extracts sender from LINE to FROM, format
 * sender@domain.com */
static void msg_parse_from (char from[], char line[])
{
	strcpy (from, strchr (line,'<') + 1);
	memset (rindex (from, '>'), 0, 1); 
}
/******************** END OF PRIVATE FUNCTIONS **********************/

/*--------------------------------------------------------------------
 int ParseFilterFile( char *fname, struct Filters *filters );

 This function reads the filterfile and adds all the filters to the
 Filters structure used by popmail to determine if a message must
 be deleted on the server.
--------------------------------------------------------------------*/
int ParseFilterFile (char *file, struct Filters *filters)
{
   FILE *fp;
   char done = 0;
   char line[256], word[256];
	action_t type; /* accept or deny filter */
/*	char *docdir = DESTDOCDIR;*/

   /* try to open the filterfile. If fails, disable filters */
   fp = fopen (file, "rt" );
   if (fp == NULL)
		return(0);

   filters->num_rules = 0;
   /* success -> read all the filter rules */
   while (!done) {
      /* get a line */
      if (fgets( line, 255, fp ) != NULL ) {
         if ((GetFirstWord (line, 255, word)) &&
				(line[0] != '#')) {
            /* look for the deny option */
            if ((strcmp (word, "deny")) == 0 ||
						strcmp (word, "accept") == 0) { /* correct keyword */
					if (strcmp (word, "accept") == 0) /* determine action to take */
						type = ACCEPT;
					else 
						type = DENY;
					/* XXX: No blanks allowed in regexps */
               if (get_last_word (line, word, sizeof (word))) {
                  /* add the filter */
						add_filter (word, filters, type);
               }
               else {
                  done = 1;
                  fprintf (stderr, _("Error parsing filterfile (%s). "
									"Read " DESTDOCDIR " and create it properly.\n"),
									file);
               }
            }
         }
      }
      if (feof(fp))
			done = 1;
   }

	fclose (fp);

	return 0;
}


/* writes all filters after BEGIN index to FILE */
/* XXX: No repeated filters control */
int WriteFilterFile (char *file, struct Filters *filters, int begin)
{
	FILE *fp;

	fp = fopen (file, "a");
	if (fp == NULL) {
		fprintf (stderr, _("Couldn't open %s file for writing\n"), file);
		exit (-1);
	}

	fprintf (fp, "\n# -- These filters were created inside interactive mode --\n");

	for (; begin < filters->num_rules; begin++) {
#ifdef DEBUG
		printf ("Writing to disk filter #%i: %s %s\n",
				begin,
				filters->rules[begin].action == DENY ? "deny" : "accept",
				filters->rules[begin].rule_text);
#endif
		fprintf (fp, "%s %s\n", 
				filters->rules[begin].action == DENY ? "deny" : "accept",
				filters->rules[begin].rule_text);
	}

	fprintf (fp, "# --------- End of interactive mode added filters --------\n");

	fclose (fp);

	return 0;
}


int add_filter (char string[], struct Filters *filters, action_t action)
{
	if (filters->num_rules < MAX_FILTERS) {
		regcomp( &filters->rules[filters->num_rules].rule, string, 
				REG_EXTENDED|REG_ICASE|REG_NOSUB );
				// cade: on the previous line: use extended regexps,
				// ignore case and don't use subs
      strcpy (filters->rules[filters->num_rules].rule_text, string);
		filters->rules[filters->num_rules].action = action;

      /*printf("Regla %d; <%s>\n", filters->num_rules,
      	filters->rules[filters->num_rules].rule_text); */
      filters->num_rules++;
	} 
	else 
		return 1;

	return 0;
}


int add_filter_generic (char from_hdr[], struct Filters *filters,
								action_t action, enum target kind)
{
	char text[128];
	char *tmp;

	if (filters->num_rules == MAX_FILTERS)
		return 1;
	
	msg_parse_from (text, from_hdr); /* extract user@domain.com */
	strcpy (filters->rules[filters->num_rules].rule_text, "^From:.*");

	switch (kind) {
		case SENDER: /* get user@ */
			tmp = rindex (text, '@');
			strcpy (tmp + 1, ".*") ; /* tmp = user@.* */
			break;
			
		case DOMAIN: /* get @domain.com */
			strncpy (text, strchr (text, '@'), 125); /* tmp = @domain.com */
			strcat (text, ".*");
			break;
			
		case FROM: /* get user@domain.com */
			strcat (text, ".*"); /* text = user@domain.com.* */
			break;

		default: /* nonexistent kind */
			return -1;
	
	}

	strncat (filters->rules[filters->num_rules].rule_text,
				text, 128 - sizeof ("From:.*"));
	filters->rules[filters->num_rules].action = action;
	filters->num_rules++;

	return 0;
}


/*int add_filter_domain (char from_hdr[], struct Filters *filters, action_t action)
{
	return 0;
}


int add_filter_sender (char from_hdr[], struct Filters *filters, action_t action)
{
	return 0;
}

*/

/* add a filter to SUBJECT. NOTE: spaces will be replaced with '.*' */
int add_filter_subject (char subject[], struct Filters *filters, action_t action)
{
	int i,j;
	char tmp[128];

	if (filters->num_rules == MAX_FILTERS)
		return 1;

	if (strlen (subject) > 127)
		subject[128] = '\0';

	memset (tmp, 0, sizeof (tmp));

	for (i=0, j=0; i < strlen (subject); i++, j++) {
		if (subject[i] == ' ') { /* replace with '.*' */
			tmp[j++] = '.';
			tmp[j] = '*';
			while (isspace (subject[i]))
				i++; /* skip blanks */

			i--; /* back 1 char (i should increment twice because the loop)*/

		} else
			tmp[j] = subject[i];
	}

	strcpy (filters->rules[filters->num_rules].rule_text, "^Subject:.*");
	strncat (filters->rules[filters->num_rules].rule_text,
			tmp, 128 - sizeof("^Subject:.*"));

	filters->rules[filters->num_rules].action = action;

	filters->num_rules++;
			
	return 0;
}


int print_filters (struct Filters *filters)
{
	int i;

	for (i=0; i < filters->num_rules; i++) {
      printf(_("Rule %d: <%s> ; type: %s\n"), i,
      	filters->rules[i].rule_text, 
			(filters->rules[i].action == ACCEPT) ? "Accept" : "Deny");
	}
		
	return 0;
}
#if 0
/*---------------------------------------------------------*/
/* filter_type: returns filter action (accept or deny) for NFITERth filter */
action_t filter_type (struct Filters *filters, int nfilter)
{
	if (nfilter > filters->num_rules)
		return -1;

	return filters->rules[nfilter].action;
}
#endif


/* returns number of filters stored in structure FILTERS */
int num_filters (struct Filters *filters)
{
	return (filters->num_rules);
}


/* 
 * Checks if a line contained in BUFFER matches any rule in FILTERS.
 * Returns:
 * 		 NULL if no match
 * 		 a pointer to the Rule that matched the string
 */
struct Rule *check_filters (char buffer[], struct Filters *filters)
{
	int i;
	int match = 0;

	for (i=0; i < filters->num_rules; i++) {
		if ((match = check_filter(&(filters->rules[i]), buffer)) == 1)
			break;
			
	}

	if (match == 1)
		return (&(filters->rules[i]));
	
	return NULL;
}

