/***********************************************************************
 * filter.h: Filtering stuff Routines for POPMail 
 * 	$Revision: 1.6 $
 *
 * 	Copyright (c) 2003 L. Alberto Gimnez Benito	<algibe@teleline.es>
 * 
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * **********************************************************************/

#ifndef _FILTER_H_
#define _FILTER_H_

#include <stdio.h>
#include <regex.h>
#include <libintl.h>
#include <sys/types.h>

#define MAX_FILTERS 9999

/*#define DEBUG*/


typedef enum { DENY, ACCEPT } action_t;

struct Rule
{
   regex_t rule;
   char    rule_text[128];
	action_t action; /* whether deny or accept */
};

struct Filters
{
   int     num_rules;
   struct  Rule rules[MAX_FILTERS];
};

/* kind of from filters */
enum target { FROM, SENDER, DOMAIN };

/*--------------------------------------------------------*/

/* Read user-specified filters in FILE and save them in FILTERS */
int ParseFilterFile (char *file, struct Filters *filters);

/* writes all filters of FILTERS to FILE. Only writes to disk filters
 * after BEGIN index */
int WriteFilterFile (char *file, struct Filters *filters, int begin);

/* add_filter: adds a filter to STRING to filters struct in the
 * proper filter file format. Return 1 if reached MAX_FILTERS */
int add_filter (char string[], struct Filters *filters, action_t action);

/* adds a filter to FROM_HDR of any KIND between FROM, SENDER and
 * DOMAIN. Returns 1 if reached MAX_FILTERS */
int add_filter_generic (char from_hdr[], struct Filters *filters,
								action_t action, enum target kind);

/* add_filter_from: adds a filter to FROM header to the
 * Filters struct (sender@domain.com). Return 1 if reached MAX_FILTERS */
#define add_filter_from(fromhdr,filters,action) \
	add_filter_generic ((fromhdr),(filters),(action),FROM)

/* add_filter_domain: adds a filter to domain of FROM header of message
 * *msg in the Filters struct (*@domain.com) */
#define add_filter_domain(fromhdr,filters,action) \
	add_filter_generic ((fromhdr),(filters),(action),DOMAIN)

/* add_filter_sender: adds a filter to the sender of From header of
 * message pointed by msg in the Filters struct (sender@*) */
#define add_filter_sender(fromhdr,filters,action) \
	add_filter_generic ((fromhdr),(filters),(action),SENDER)

/* add_filter_subject: Adds a filter to the Subject header in a proper
 * format to filter file syntax in the filters struct */
int add_filter_subject (char subject[], struct Filters *filters, action_t action);

/* 
 * Checks if a line contained in BUFFER matches any rule in FILTERS.
 * Returns:
 * 		 NULL if no match
 * 		 a pointer to the Rule that matched the string
 */
struct Rule *check_filters (char buffer[], struct Filters *filters);

/* print_filters: Prints all filters of filters struct */
int print_filters (struct Filters *filters);

/*----------------------------------------------------------*/

#if 0
/* filter_type: returns filter action (accept or deny) for NFITERth filter.
 * Returns -1 if no NFILTERth filter exist */
action_t filter_type (struct Filters *filters, int nfilter);
#endif

/* returns number of filters stored in structure FILTERS */
int num_filters (struct Filters *filters);

#endif /* _FILTER_H_ */
