/***********************************************************************
 * message.c:  Message stuff Routines for POPMail
 * 	$Revision: 1.8 $
 *
 * 	Copyright (c) 2003 L. Alberto Gimnez Benito	<algibe@teleline.es>
 * 
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * **********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "message.h"

static const char rcsid[]="$Id: message.c,v 1.8 2004/09/09 11:53:29 alberto Exp $";

int ExtractMessageHeader(struct Message *msg);
	
/*****************************************************
 *         Construction routines
 ****************************************************/

/* msg_create: creates a void message struct */
struct Message * msg_create (void)
{
	struct Message *msg;

	msg = (struct Message *) malloc (sizeof (struct Message));

   strcpy (msg->from, "none");
   strcpy (msg->subject, "none");
	strcpy (msg->date, "none");
	strcpy (msg->header, "none");

	msg->nbytes = 0;
	msg->deleted = 0;

	return msg;
}

/* msg_destroy: deletes a message struct */
int msg_free (struct Message *msg)
{
	free (msg);
	msg = NULL;

	return 0;
}
	
/* msg_set_deleted: marks message as deleted */
int msg_set_deleted (struct Message *msg)
{
	msg->deleted = 1;
	return 0;
}

/* msg_clear_deleted: marks message as not deleted */
int msg_clear_deleted (struct Message *msg)
{
	msg->deleted = 0;
	return 0;
}

/* msg_set_header: updates header of msg to string */
int msg_set_header (struct Message *msg, char hdr[])
{
   strncpy (msg->header, hdr, 4192);
	msg->header[4192] = '\0'; /* assert end of string */
   strcpy (msg->header + strlen (msg->header)-4, "\n.\n\0");
   msg->deleted = 0;

	ExtractMessageHeader (msg);

	return 0;
}

/* msg_set_size: sets msg size */
int msg_set_size (struct Message *msg, int size)
{
	if (size < 0)
		msg->nbytes = 0;
	else
		msg->nbytes = size;
	return 0;
}


/* msg_set_date: sets date of msg to DATE */
int msg_set_date (struct Message *msg, char date[])
{
	strncpy (msg->date, date, 256);
	msg->date[255] = '\0';
	
	return 0;
}


/* msg_set_subject: sets msg subject */
int msg_set_subject (struct Message *msg, char subject[])
{
	strncpy (msg->subject, subject, 256);
	msg->subject[255] = '\0';
	 
	return 0;
}


/* msg_parse_from: extracts sender from STRING to FROM, format
 * sender@domain.com */
/*int msg_parse_from (char from[], char line[])
{
	return 0;
}
*/

/*******************************************
 * 		Query Routines
 ******************************************/

/* msg_size: returns the size in bytes of message pointed by *msg */
int msg_size (struct Message *msg)
{
	return msg->nbytes;
}

/* msg_header: returns a string containing the full headers of msg */
char *msg_header (struct Message *msg)
{
	return msg->header;
}

/* msg_from: returns a string containing the From Header */
char *msg_from (struct Message *msg)
{
	return msg->from;
}

/* msg_date: returns a string containing the date of msg */
char *msg_date (struct Message *msg)
{
	return msg->date;
}

/* msg_subject: returns the Subject of msg */
char *msg_subject (struct Message *msg)
{
	return msg->subject;
}

/* msg_deleted: returns 1 if msg is marked for deletion, 0 otherwise */
int msg_deleted (struct Message *msg)
{
	return (msg->deleted);
}

/* **************** *
 * Private routines *
 * **************** */

/* ExtractMessageHeader: extracts from the header field the From,
 * Subject and Date. */
int ExtractMessageHeader (struct Message *msg)
{
   int posicion=0;
   char linea[2048];

   do
   {
      /* We now extract the different lines of the header... */
      posicion = ExtractLine (msg->header, 2047, posicion, linea);
      linea[255] =  '\0';

      /* ... and search for known header fields (From, Date...). */
      if( strstr(linea, "From: ") == linea )
			strcpy (msg->from, linea+strlen("From: "));
		/* not case sensitive */
		else if (strstr (linea, "FROM: ") == linea)
			strcpy (msg->from, linea+strlen ("FROM: "));

      else if( strstr(linea, "Subject: ") == linea )
			strcpy (msg->subject, linea+strlen("Subject: "));
      else if( strstr(linea, "SUBJECT: ") == linea )
			strcpy (msg->subject, linea+strlen("SUBJECT: "));

      else if( strstr(linea, "Date: ") == linea )
			strcpy (msg->date, linea+strlen("Date: "));
      else if( strstr(linea, "DATE: ") == linea )
			strcpy (msg->date, linea+strlen("DATE: "));
   }
   while( linea[0] != '.' );

   return(1);    /* All done! */
}
