/***********************************************************************
 * message.h:  Message stuff Routines for POPMail
 * 	$Revision: 1.4 $
 *
 * 	Copyright (c) 2003 L. Alberto Gimnez Benito	<algibe@teleline.es>
 * 
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * **********************************************************************/

#ifndef _MESSAGE_H_
#define _MESSAGE_H_

#include "util.h"

struct Message
{
   char    header[4192]; 
   int     nbytes; 
   char    from[256];
   char    date[256]; /* is it ever used? */
   char    subject[256];
   char    deleted; 
};

/*****************************************************
 *         Construction routines
 ****************************************************/
/* msg_create: creates a void message struct */
struct Message * msg_create (void);

/* msg_destroy: deletes a message struct */
int msg_free (struct Message *msg);

/* msg_set_deleted: marks message as deleted */
int msg_set_deleted (struct Message *msg);

/* msg_clear_deleted: marks message as not deleted */
int msg_clear_deleted (struct Message *msg);
	
/* msg_set_header: updates header of msg to string */
int msg_set_header (struct Message *msg, char hdr[]);

/* msg_set_size: sets msg size */
int msg_set_size (struct Message *msg, int size);
      
/* msg_set_date: sets date of msg to DATE */
int msg_set_date (struct Message *msg, char date[]);
      
/* msg_set_subject: sets msg subject */
int msg_set_subject (struct Message *msg, char subject[]);


/*******************************************
 * 		Query Routines
 ******************************************/
/* msg_size: returns the size in bytes of message pointed by *msg */
int msg_size (struct Message *msg);

/* msg_header: returns a string containing the full headers of msg */
char *msg_header (struct Message *msg); 

/* msg_from: returns a string containing the From Header */
char *msg_from (struct Message *msg);

/* msg_date: returns a string containing the date of msg */
char *msg_date (struct Message *msg);

/* msg_subject: returns the Subject of msg */
char *msg_subject (struct Message *msg);

/* msg_deleted: returns 1 if msg is marked for deletion, 0 otherwise */
int msg_deleted (struct Message *msg); 

#endif /* _MESSAGE_H_ */
