/*====================================================================
  popmail.c ->   That's my POP3-email-checker-plus-spam-eraser: just
                 a way to browse the contents of your mailboxes and
                 delete emails you don't want to fetch. This program
                 will not fetch the emails (use fetchmail, jfpop or
                 fetchpop to do this), it just applies some rules to
                 the messages IN THE MAILBOX in order to allow you
                 to delete some emails before receiving them (you
                 could, as an example, delete emails with a given
                 FROM: (sender) header so that forget about some
                 common spammers). Take a look at the README to learn
                 about what you can do with POPmail.
  $Revision: 2.11 $

  I have rewritten and used some code of mailcheck v0.1 by Steven
  Radack (steve@lconn.net), and JFPOP from *puntero_loco and Ferkys
  (of course, with permission of the original coders :)

             (c) 2000 Santiago Romero AkA NoP / Compiler
                       santiago.romero@iname.com

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ====================================================================*/
#include "popmail.h"
#include "cfgfile.h"
#include "interface.h"
#include "filter.h"

#define _(string) gettext(string)
#define N_(string) (string)

static const char rcsid[]="$Id: popmail.c,v 2.11 2004/09/09 11:53:29 alberto Exp $";

int opt_dummy = 0;
struct  Server **servidores;
t_mode    mode;
struct  Filters filters;

void ParseArgs(int argc, char **argv, char *configfile );
void print_help(void);

/*---- main function --------------------------------------------------*/
int main(int argc, char *argv[])
{
   int     i, j;
   struct  stat buf;
   int     num_servers=1;
   char    configfile[256], logfname[256];
	char	filterfile[256];
   FILE    *cfgfile, *logfile;
   int     maxsize = -1, key;
	int return_value; /* return value depending on wether we have mail or not */ 	
	
	setlocale(LC_ALL,"");
	bindtextdomain("popmail",LOCALEDIR);
	textdomain("popmail");

   ParseArgs (argc, argv, configfile);
   if( mode != SILENT )
   {
   	printf(_("POPMail - (c) 2000 Santiago Romero - "
					"(c) 2003 L. Alberto Gimnez\n"
					"Published under the GNU License.\n\n"));
      if ( opt_dummy )          
        {
        // cade: this should be printed in any mode I think...
        printf(_("warning: dummy mode: messages will not be deleted.\n"));
        }
   }

   /* Check file permissions. Should be 0600 (only owner can read and write) */
   if( stat( configfile, &buf ) == 0 )
   {
      if( buf.st_mode != (S_IRUSR | S_IWUSR | S_IFREG) )
      {
         printf(_("> Your .popmailrc file has wrong permissions. People could\n"
						"  read your users & passwords if you don't change it to 0600.\n"
						"  Do you want popmail to change it to 0600 for you? (by\n"
						"  executing \"chmod 0600\") (y/n) "));
         fflush(stdin);
         key = getchar();
         if( key=='y' || key=='Y' || key=='s' || key=='S' )
         {
            printf(_("\n> Now I'm going to exec \"chmod 0600 .popmailrc\" to set"
							"secure permissions.\n"
							"  This way nobody but you will be able to read your "
							"users/passwords: "));
            chmod( configfile, S_IRUSR | S_IWUSR | S_IFREG );
            printf(_("done.\n\n"));
         }
         else
         {
            printf(_("\n> Please change the permissions to 0600 (chmod 0600"
							".popmailrc) before\n"
							"  executing this program, so that people can't read"
							"your setup/config file.\n\n"));
            return(-4);
         }
      }
   }
   else /* file does not exist? */
   {
       if( errno == ENOENT )
           CreateCFGFile( configfile );
	   else
           printf(_("\n> stat() returned %d <\n"), errno );
       return(-1);
   }

   cfgfile = fopen( configfile, "rt" );
   if( cfgfile == NULL )
   {
       CreateCFGFile( configfile );
       return(-1);
   }
   num_servers = ParseFileForNumServers( cfgfile );

   /*--- Get memory for the server list: ------------------------*/
   servidores = (struct Server **)
           malloc( sizeof(struct Server *)*num_servers);
   if( servidores == NULL )
       error(_("Error getting memory for the server list.\n"));

   for( i=0; i<num_servers; i++ )
   {
       servidores[i] = server_create ();
       if( servidores[i] == NULL )
       {
          for( j=0; j<i; j++ )
             server_free(servidores[j]);
          free(servidores);
          error(_("Error getting memory for a server struct.\n"));
       }
   }
   /*------------------------------------------------------------*/

   ParseFileForServers( cfgfile, num_servers, servidores );
   ParseFileForOptions( cfgfile, logfname, &maxsize, &filters, filterfile);

   logfile = fopen( logfname, "at" );
   if( mode == VERBOSE  )
   {
       if( maxsize > 0 )
          printf(_(" Max Message Size = %d bytes.\n"), maxsize);
       else
          printf(_(" Filtering messages by size disabled.\n"));
       if( filters.num_rules > 0 )
          printf(_(" Filtering capabilites enabled (%d entries).\n"), filters.num_rules );
       else
          printf(_(" Filtering capabilities disabled.\n"));
       if( logfile == NULL )
       {
          printf(_(" Logging capabilites disabled. \n"));
       }
       else
          printf(_(" Logging capabilites enabled. \n"));
   }

   fclose( cfgfile );

   /* This is the main function: We log into each server and test
      filter/size rules, showing the results on the screen. */
	return_value = 1; /* if we have no mail, return nonzero */
   for( i=0; i<num_servers; i++ ) {
      ReadServer( servidores[i], &filters, maxsize, logfile );
		if ((server_kept (servidores[i]) > 0) && (return_value == 1))
			return_value = 0;
	}

	if (!return_value && mode == INTERACTIVE) /* there is mail, browse mailbox */
		return_value = iface_init (num_servers, filterfile);
	else if (mode == INTERACTIVE) /* no mail remaining */
		printf (_("No mail remaining, no interactive mode allowed.\n"));
	
   /*--- Free memory allocated for the server list: -------------*/
	for (i=0; i < num_servers; i++) {
		real_deletion (servidores[i]);
   	POPDisconnect (servidores[i]);
      free(servidores[i]);
	}
   free( servidores );
   /*------------------------------------------------------------*/

   /*--- Free regexp allocated with regcomp() ---------------------*/
   for( i=0; i<filters.num_rules; i++ )
      regfree( &filters.rules[filters.num_rules].rule );

   if( logfile != NULL )
      fclose(logfile);

   if( mode != SILENT )
      printf("\n");

	return (return_value); 
}


/*----------------------------------------------------------------
  void ParseArgs(int argc, char **argv, char *mode, char *cfgfile );
  Analizes the program options using getopt (man 1 getopt) and
  sets the path to the configuration file
 ----------------------------------------------------------------*/
void ParseArgs(int argc, char **argv, char *configfile)
{
 
   int option_index;
   int c;
   static struct option long_options[] ={
    {"help", 0, 0, 'h'},
	 {"version",0,0,'V'},
    {"verbose", 0, 0, 'v'},
    {"silent", 0, 0, 's'},
    {"normal", 0, 0, 'n'},
    {"rebuildconfig", 0, 0, 'r'},
    {"dummy", 0, 0, 'd'},
    {"file", 1, 0, 'f'},
    {"interactive", 0, 0, 'i'},
    {0, 0, 0, 0}
   };
   
   extern char *optarg;
   extern int optind, opterr, optopt;
   struct  stat buf;
   char    *home;
	char version[] = VERSION;

   mode = NORMAL;
   c = 0;
	
   /* look for the HOME directory and the config file */
   home = getenv("HOME");
   sprintf(configfile, "%s/.popmail/popmailrc", home ); /* default */

   while( c != -1)
   {
      option_index = 0;
      c = getopt_long(argc, argv, "hVvsnrdf:i", long_options, &option_index);
      
      if( c != -1 )
      switch(c) {
   	   case 'h': 
				print_help ();
				exit (0);

   	   case '?':
				print_help ();
            exit (1);

			case 'V':
   			printf(_("POPMail - (c) 2000 Santiago Romero - "
							"(c) 2003 L. Alberto Gimnez\n"
							"Published under the GNU License.\n\n"
							"Version %s\n\n"), version);
				exit(0);				

			case 'i': mode = INTERACTIVE;break;
      	case 'v': mode = VERBOSE; break;
      	case 's': mode = SILENT; break;
      	case 'n': mode = NORMAL; break;

			case 'f':
				strcpy (configfile, optarg); /* overwrite the default */
				break;

      	case 'r':
   			printf(_("POPMail - (c) 2000 Santiago Romero - "
							"(c) 2003 L. Alberto Gimnez\n"
							"Published under the GNU License.\n\n"));
						
            if( stat( configfile, &buf ) == 0 )
            {
                if( remove(configfile) != 0 )
                {
                   printf(_("Couldn't remove %s\n"), configfile );
                }
            }
            CreateCFGFile( configfile );
            exit(0);
            break;

      	case 'd'  :
              opt_dummy = 1;
              break;


			default: /* unexpected */
				  abort();
      }
   }

}

void print_help(void)
{
	char *docdir = DESTDOCDIR;
   printf(_("POPMail - (c) 2000 Santiago Romero - (c) 2003 L. Alberto "
				"Gimnez\n"
				"Published under the GNU License.\n\n"
 				"POP3 email checker and anti-spam filtering. Read\n"
				"%s and edit your config file to configure\n"
				"popmail properly.\n\n"
	"  Usage:\tpopmail [options]\n\n"
	"  Options:  -s, --silent           Do not report info to the screen.\n"
	"            -n, --normal           Report some info to the screen.\n"
	"            -v, --verbose          Be verbose reporting info.\n"
	"            -r, --rebuildconfig    Creates a new config file.\n"
	"            -h, --help             This help.\n"
	"            -V, --version          Prints POPMail version.\n"
	"            -d, --dummy            Do not delete messages (debug or test).\n"
	"            -f, --file=FILE        Use FILE as config file.\n"
	"            -i, --interactive      Enter interactive mode after apply\n"
	"                                   the filters\n\n"), docdir);

}
