/***********************************************************************
 * server.c: Server stuff Routines for POPMail
 * 	$Revision: 1.15 $
 *
 * 	Copyright (c) 2003, 2004  L. Alberto Gimnez Benito
 * 	<algibe@teleline.es>
 * 
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * **********************************************************************/

#include "server.h"
#include "popmail.h"
#include "popsockets.h"

#define _(string) gettext (string)
#define N_(string) (string)

static const char rcsid[]="$Id: server.c,v 1.15 2004/09/10 19:39:38 alberto Exp $";

int POPConnect( struct Server *);
int GetMessageHeader( struct Server *, int);
int DeleteMessage( struct Server *, int);
int DeleteServerMessage( struct Server *, int );
void POPDisconnect( struct Server *);
int POPLogin( struct Server *);

/* *******************************
 * 	Construction routines		*
 * *******************************/

/* server_create: creates a void server list structure with room for
 * NUM_SERVERS servers */
struct Server *
server_create(void)
{
	struct Server *server;
	
   server = (struct Server *)
           malloc (sizeof (struct Server));
	
   if (server != NULL) {
		strcpy(server->mailserver, "localhost");
		strcpy(server->username,   "luser");
		strcpy(server->password,   "luser");
		server->puerto = 110;
		server->num_msgs = 0;
	}

	return server;
}
	
/* server_set_user: sets USER as username for server SERVER */
int server_set_user (struct Server *server, char user[])
{
	strncpy (server->username, user, 32);
	return 0;
}

/* server_set_pass: sets PASS as password for server SERVER*/
int server_set_pass (struct Server *server, char pass[])
{
	strncpy (server->password, pass, 32);
	return 0;
}

/* server_set_name: sets NAME as server hostname for SERVER*/
int server_set_name (struct Server *server, char name[])
{
	strncpy (server->mailserver, name, 128);
	return 0;
}

/* server_set_size: sets SERVER messages total size to SIZE */
int server_set_size (struct Server *server, int size)
{
	server->totalsize = size;
	return 0;
}

/* server_set_size: sets SERVER messages total size to SIZE */
int server_set_port (struct Server *server, int port)
{ 
	server->puerto = port;
	return 0;
}

/*--------------------------------------------------------------------
 int ReadServer( struct Server *servidor, mode, maxsize );

 This function receives a correctly (I hope) filled Server struct
 with the pop3 server address, the POP3 port, the username and the
 password to log in and obtains all the info about that server (the
 number and size of messages, their headers, and so on...
 This function returns the number of messages or -1 if it failed.
--------------------------------------------------------------------*/
int ReadServer(struct Server *servidor, struct Filters *filters,
                int maxsize, FILE *log )
{
	char buffer[4096], token[32], tmp[4096];
	int i, j;
	struct timeb pt;
	int accepted = 0; /* message matched an accept rule */

	/* Do login and continue only of returns == 0 */
	if ( POPLogin( servidor) != 0 )
		return -1;
	else
	{
		/* USER and PASS accepted -> go for it! */
		if( mode == VERBOSE )
			printf(_("\n- Connected with POP3 server.\n"));

		servidor->num_msgs = 0;
		/* Get the message list (number and size): */
		Sendf( servidor->sockfd, 0, "STAT\r\n");
		GetLineFromSocket( servidor->sockfd, buffer, sizeof(buffer));
		CheckBuffer(buffer);

		strcpy(tmp, buffer); /* why? */
		GetnWord( buffer, 2, token, sizeof (token));
		servidor->num_msgs = atoi(token);
		servidor->kept_msgs = servidor->num_msgs;

		GetnWord( tmp, 3, token, sizeof (token) );
		servidor->totalsize = atoi(token);

		if( mode != SILENT )
			printf(_("  - The user %s in %s has %d message(s)"),
					servidor->username, servidor->mailserver,
					servidor->num_msgs);
		if( mode == VERBOSE ) printf(".\n");

		/*--- Get memory for the message list: ----------------------*/
		servidor->msgs = (struct Message **)
			malloc( sizeof(struct Message *)*servidor->num_msgs);
		if( servidor->msgs == NULL )
		{
			error(_("Error getting memory for the message list.\n"));
			POPDisconnect (servidor);
		}

		for( i=0; i<servidor->num_msgs; i++ )
		{
			servidor->msgs[i] = msg_create ();
			if( servidor->msgs[i] == NULL )
			{
				for( j=0; j<i; j++ )
					msg_free(servidor->msgs[j]);
				free(servidor->msgs);
				error(_("Error getting memory for a message struct.\n"));
				POPDisconnect (servidor);
			}
		}
		/*--------------------------------------------------------------*/

		for( i=0; i<servidor->num_msgs; i++ )
		{

			/* first of all execute a LIST command to get msg size */
			snprintf( tmp, sizeof(tmp), "LIST %d\r\n", i+1);
			Sendf( servidor->sockfd, 0, tmp);
			GetLineFromSocket( servidor->sockfd, buffer, sizeof(buffer));
			CheckBuffer(buffer);
			GetnWord( buffer, 3, token, sizeof (token));
			msg_set_size (servidor->msgs[i], atoi (token));

			/* now extract it's header and tell user info about the msg */
			GetMessageHeader (servidor, i);

			/* Display just 40 characters of the subject */
			if( mode == VERBOSE )
			{
				if (strlen(msg_subject (servidor->msgs[i])) < 40 )
				{
					printf(_("    > %2d: %-50s (%d bytes).\n"), i+1,
							msg_subject (servidor->msgs[i]),
							msg_size (servidor->msgs[i]));
				}
				else
				{
					strcpy(tmp, msg_subject (servidor->msgs[i]));
					tmp[39] = '\0';
					printf(_("    > %2d: %-50s (%d bytes).\n"), i+1, tmp,
							msg_size (servidor->msgs[i]));
				}
				printf(_("          From: %s\n"), msg_from (servidor->msgs[i]));
			}

			if (mode != SILENT) 
				putchar('.');

			fflush(stdout);
		}

		if( mode == VERBOSE )
			printf(_("    Total size: "));
		if( mode != SILENT )
			printf(_(" (%d bytes).\n"), servidor->totalsize );

		/*----> Now comes the filter stuff <----------------------------*/
		for( i=0; i < servidor->num_msgs; i++ )
		{
			accepted = 0;
			/* count message lines */
			//numlines = CountLines (msg_header (servidor->msgs[i]));

			/* first check if message is larger than the max size */
			if( msg_size (servidor->msgs[i]) > maxsize && maxsize != -1 )
			{
				msg_set_deleted (servidor->msgs[i]);

				if( mode != SILENT )
					printf(_("    Message %3d deleted: Size (%d) > MAXSIZE (%d).\n"),
							i+1, msg_size (servidor->msgs[i]), maxsize );

				/* log if needed */
				if( log!= NULL )
				{
					ftime(&pt);
					fprintf(log,
							_("POPMail - Account: %s on %s at %s > Msg"\
								"%d deleted: %d bytes > MAXSIZE (%d).\n > From: %s\n > "\
								"Subject: %s.\n\n"),
							servidor->username, servidor->mailserver,
							ctime(&(pt.time)), i+1, msg_size (servidor->msgs[i]),
							maxsize, msg_from (servidor->msgs[i]),
							msg_subject (servidor->msgs[i]));
				}

			}
			/* now check the filters themselves */
			else {
				struct Rule *match;

#if 0
				offs = ExtractLine (msg_header (servidor->msgs[i]),
						255, offs, tmp );
#endif

				if ((match = check_filters(msg_header(servidor->msgs[i]),
									filters)) != NULL) {
#ifdef DEBUG
					printf("Math: Rule %s\n", match->rule_text);
					printf("Match action: %d\n", match->action);
#endif
					if (match->action == DENY) {
						msg_set_deleted(servidor->msgs[i]);

						if (mode != SILENT) {
							printf(_("    Message %3d deleted: Deny %s .\n"),
									i+1, match->rule_text );
						}

						if( log!= NULL ) {
							ftime(&pt);
							fprintf(log,
									_("POPMail - Account: %s on %s at %s > Msg"
										"%3d deleted (%d). Deny %s\n > From: %s\n > "
										"Subject: %s.\n\n"),
									servidor->username, servidor->mailserver,
									ctime(&(pt.time)), i+1,
									msg_size (servidor->msgs[i]),
									match->rule_text,
									msg_from (servidor->msgs[i]),
									msg_subject (servidor->msgs[i]));
						}
					} else if (match->action == ACCEPT) {
						accepted = 1;
					}

					if (msg_deleted (servidor->msgs[i]) ||accepted)
						continue; /* if deleted or 'filtered' as accepted
								  don't want to look more rules */
				}

				if (msg_deleted (servidor->msgs[i]) || accepted) 
					continue; /* if deleted or 'filtered' as accepted
							  don't want to look more lines */
			}


		}
	}
	/*--------------------------------------------------------------*/

	/*		if (mode != INTERACTIVE){  don't want to lose msg information
	 * if enter in interactive mode */
	/*--- Free memory allocated for the message list: --------------
	  for( i=0; i<servidor->num_msgs; i++ )
	  msg_free (servidor->msgs[i]);
	  free (servidor->msgs);
	  --------------------------------------------------------------

	  POPDisconnect (servidor);
	  }*/
	return (servidor->kept_msgs);
}

	
/* real_deletion: deletes all messages marked for deletion in all
 * servers */
int real_deletion (struct Server *server)
{
	int message;

	if (opt_dummy) {
		printf (_("Dummy mode: no messages were deleted\n"));
		return 0;
	}
	for (message = 0; message < server_msgs (server); message++)
		if (msg_deleted (server->msgs[message])) /* marked */
			DeleteMessage (server, message);

	return 0;
}


/* server_free: deletes a server list structure */
int server_free (struct Server *server)
{
	int i;
	
	for (i = 0; i < server->num_msgs; i++)
		msg_free (server->msgs[i]);
	
	free (server);
	server = NULL;
	
	return 0;
}

/*********************************
 * 	Query routines					*
 ********************************/
/* server_user: returns username of server number NSERVER in the
 * SERVERLIST */
char * server_user (struct Server *server)
{
	char *user;

	user = strdup (server->username);
	return user;
}

/* server_pass: returns password  of server number NSERVER in the
 * SERVERLIST */
char * server_pass (struct Server *server)
{
	char *pass;

	pass = strdup (server->password);
	return pass;
}

/* server_name: returns hostname of server number NSERVER in the
 * SERVERLIST */
char * server_name (struct Server *server)
{
	char *name;

	name = strdup (server->mailserver);
	return name;
}
	

/* server_msgs: returns number of messages of server number 
 * NSERVER in the SERVERLIST */
int server_msgs (struct Server *server)
{
	return server->num_msgs;
}

/* server_kept: gets number of kept messages in SERVER */
int server_kept (struct Server *server)
{
	return server->kept_msgs;
}

/* server_sockfd: returns descriptor of server number NSERVER in the
 * SERVERLIST */
int server_sockfd (struct Server *server, int sockfd)
{
	return server->sockfd;
}

/* server_size: returns totalsize of server number NSERVER in the
 * SERVERLIST */
int server_size (struct Server *server, int size)
{
	return server->totalsize;
}

/********************
 * Private routines *
 * *****************/

/*--------------------------------------------------------------------
 int POPConnect( struct Server *servidor )

 This function connects the socket to the POP3 server.
--------------------------------------------------------------------*/
int POPConnect( struct Server *servidor )
{
   struct sockaddr_in addr;
   struct hostent *he = gethostbyname(servidor->mailserver);

   if (!he)
   {
      herror(servidor->mailserver);
      return(-1);
   }

   memset( (char *)&addr, 0, sizeof(struct sockaddr_in) );
   addr.sin_family = AF_INET;
   memcpy( &(addr.sin_addr.s_addr), he->h_addr, he->h_length);
   addr.sin_port=htons(servidor->puerto);
   
   if ( connect(servidor->sockfd, (struct sockaddr *)&addr,
                     sizeof(struct sockaddr) ) < 0)
   {
      Perror("connect()");
      return (-1);
   }
   return(1);
}


/*--------------------------------------------------------------------
 int POPLogin( struct Server *servidor )

 This function logins in the POP3 server. 0=OK
--------------------------------------------------------------------*/
int POPLogin( struct Server *servidor)
{
   int bytes;
   char buffer[4096], cmd[32];

   /* First we create a socket... */
   if ( (servidor->sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
      perror("socket()");

   /* and then we connect it to the specified server... */
   if (POPConnect(servidor) < 0)
      return(-1);

   bytes = read(servidor->sockfd,buffer,sizeof(buffer));

   /* send username... */
   snprintf(cmd, sizeof(cmd), "USER %s\r\n", servidor->username);
   write( servidor->sockfd, cmd, strlen(cmd) );
   bytes = read( servidor->sockfd, buffer, sizeof(buffer));

   /* send password... */
   snprintf( cmd, sizeof(cmd), "PASS %s\r\n", servidor->password);
   write( servidor->sockfd, cmd, strlen(cmd) );
   bytes = read( servidor->sockfd, buffer, sizeof(buffer));
   buffer[bytes] = 0;

   /* first character of server reply will be + or - */
   if (buffer[0] == '-')
   {
      printf(_("Password for %s is wrong or mailbox already locked.\n"), servidor->username);
      POPDisconnect (servidor);
      close(servidor->sockfd);
      return(-2);
   }

   return(0);
}


/*--------------------------------------------------------------------
 int POPDisconnect( struct Server *servidor )

 This function disconnects the socket from the POP3 server.
 It returns the first character of the server message. It
 can be '+' for OK, and '-' for ERROR.
--------------------------------------------------------------------*/
void POPDisconnect( struct Server *servidor)
{
   int bytes;
   char buffer[4096];

   /* now we send the QUIT command and wait for +OK */
   Sendf( servidor->sockfd, 0, "QUIT\r\n");
   bytes = read( servidor->sockfd, buffer, sizeof(buffer));
   buffer[bytes] = 0;

   if ( buffer[0] == '+' ) {
      if( mode==VERBOSE ) printf(_("- Disconnected from POP3 server.\n"));
	}
   else {
      if( mode==VERBOSE ) printf(_("- Error closing connection with POP3 server.\n"));
	}

}


/*--------------------------------------------------------------------
 int GetMessageHeader( struct Server *servidor, int num_msg );

 This function receives a server with an already established socket
 and retrieves the <num_msg> message header from it (TOP n 0).
--------------------------------------------------------------------*/
int GetMessageHeader( struct Server *servidor, int num_msg)
{
   char cmd[128], buffer[1024];
   char header[8192] ;
   int done=0, bytes_read = 0;

   header[0] = buffer[0] = '\0';

   /* send the "TOP 0" command to retrieve the message header */
   snprintf(cmd, sizeof(cmd), "TOP %d 0\r\n", num_msg+1 );
   Sendf( servidor->sockfd, 0, cmd);

   /* Read the "+OK <size> octects" message */
   GetLineFromSocket( servidor->sockfd, buffer, sizeof(buffer));

   if (buffer[0] != '+')
   {
      printf("ERROR: %s\n", buffer );
      POPDisconnect (servidor);
      return(-1);
   }

   /* now read the message header into the msg struct: */
   while( !done )
   {
       GetLineFromSocket( servidor->sockfd, buffer, sizeof(buffer));
       bytes_read += strlen( buffer );
       if( buffer[0] == '.' )
           done = 1;
       else if (sizeof (header) > bytes_read)
           strncat( header, buffer, sizeof (header) - bytes_read);
		 else /* have read more bytes tan header long. Stop saving on header */
			 /* XXX: it happens only in rare cases (very long long
			  * recipients lists, etc), could be handled in other way? */
			 continue;
   }

	msg_set_header (servidor->msgs[num_msg], header);
	
   return(1);
}


/*--------------------------------------------------------------------
 int DeleteMessage( struct Server *servidor, int num_msg );

 Deletes a given message from the server (returns 1=OK, -1=ERR).
--------------------------------------------------------------------*/
int DeleteMessage( struct Server *servidor, int num_msg)
{
   char cmd[32], buffer[128];

   if( opt_dummy )
     {
     // cade: this should be printed in any mode I think...
     printf(_("dummy mode: message skipped.\n"));
     return(1);
     }
   
   /* send the "DELE n" command to delete the selected message */
   snprintf(cmd, sizeof(cmd), "DELE %d\r\n", num_msg+1 );
   Sendf( servidor->sockfd, 0, cmd);

   /* Read the "+OK" message */
   GetLineFromSocket( servidor->sockfd, buffer, sizeof(buffer));
   if (buffer[0] != '+')
   {
       if( mode != SILENT )
           printf(_("Error: %s\n"), buffer );
       return(-1);
   }

   /* Now we eliminate the message from the structure, by marking
	 * the message as deleted. We will free mem later */
   msg_set_deleted (servidor->msgs[num_msg]);

   return(1);
}


