/***********************************************************************
 * server.h: Server stuff Routines for POPMail
 * 	$Revision: 1.3 $
 *
 * 	Copyright (c) 2003 L. Alberto Gimnez Benito	<algibe@teleline.es>
 * 
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * **********************************************************************/

#ifndef _SERVER_H_
#define _SERVER_H_

#include <sys/timeb.h>
#include <time.h>
#include <netdb.h>

#include "message.h"
#include "filter.h"

struct Server
{
   char    username[32];
   char    password[32];
   char    mailserver[128];
   int     puerto;
   int     num_msgs;		/* total number of msgs */
	int     kept_msgs;   /* remaining messages */
   int     sockfd;
   int     totalsize;
   struct  Message **msgs; /* message list */
};


void POPDisconnect( struct Server *);

/* *******************************
 * 	Construction routines		*
 * *******************************/

/* server_create: inits a void server structure with nonsense data */
struct Server * server_create (void);
	
/* server_set_user: sets USER as username for server SERVER */
int server_set_user (struct Server *server, char user[]);

/* server_set_pass: sets PASS as password for server SERVER*/
int server_set_pass (struct Server *server, char pass[]);

/* server_set_name: sets NAME as server hostname for SERVER*/
int server_set_name (struct Server *server, char name[]);

/* server_set_size: sets SERVER messages total size to SIZE */
int server_set_size (struct Server *server, int size);

/* server_set_size: sets SERVER messages total size to SIZE */
int server_set_port (struct Server *server, int port);

/* Apply all filters to SERVER */
int ReadServer (struct Server *server,
						struct Filters *filters,
						int maxsize, /* max message size */
						FILE *log ); /* where will log filters */

/* Performs all real deletion stuff */
int real_deletion (struct Server *server);

/* deletes message number NMESS in SERVER */
int DeleteMessage (struct Server *server, int nmess);

/* deletes all messages in SERVER and frees mem */
int server_free (struct Server *server);


/*********************************
 * 	Query routines					*
 ********************************/

/* server_user: returns username of server number NSERVER in the
 * SERVERLIST */
char * server_user (struct Server *server);

/* server_pass: returns password  of server number NSERVER in the
 * SERVERLIST */
char * server_pass (struct Server *server);

/* server_name: returns hostname of server number NSERVER in the
 * SERVERLIST */
char * server_name (struct Server *server);

/* server_msgs: returns number of messages of server number 
 * NSERVER in the SERVERLIST */
int server_msgs (struct Server *server);

/* server_kept: gets number of kept messages in SERVER */
int server_kept (struct Server *server);

/* server_sockfd: returns descriptor of server number NSERVER in the
 * SERVERLIST */
int server_sockfd (struct Server *server, int sockfd);

/* server_size: returns totalsize of server number NSERVER in the
 * SERVERLIST */
int server_size (struct Server *server, int size);

#endif /* _SERVER_H_ */
