/*====================================================================
  util.c -> Auxiliary routines
  $Revision: 1.10 $

              (c) 2000 Santiago Romero AkA NoP / Compiler
                         sromero@unix-shells.com

				(c) 2003 L. Alberto Gimnez
					<algibe@teleline.es>

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  Please read the README.* files to know how popmail works.
 ====================================================================*/

#include "util.h"

#define _(string) gettext(string)
#define N_(string) (string)

static const char rcsid[]="$Id: util.c,v 1.10 2004/09/09 11:53:29 alberto Exp $";

/*--------------------------------------------------------------------
 void CheckBuffer( char * buffer );

 This function checks the output of an read() command received from
 the POP3 server (firts character will be + or - for right/wrong
 command received (+OK, -ERR).
--------------------------------------------------------------------*/
void CheckBuffer(char *buffer)
{
   if (!((buffer[0] == '+') &&
        (buffer[1] == 'O') &&
        (buffer[2] == 'K')))
   {
      printf(_("Couldn't init transfer.\n"));
      printf(_("Message from server: %s\n"), buffer);
      exit(-1);
   }

}


/* Counts number of white-space delimited words of STRING */
int count_words (char *string)
{
	int count; /* word */
	int index; /* character */
	int in_word = 0; /* inside a word */
	size_t length = strlen(string);

	
	for (count = 0, index = 0; index < length; index++) {
		if (isspace (string[index]))
			in_word = 0;
		else if (!in_word) {
			in_word = 1;
			count++;
		}
	}

	return count;
}


/*--------------------------------------------------------------------
GetnWord: returns the n'th word (COUNT characters long at most) 
of the sentence pointed by s. 
--------------------------------------------------------------------*/
int GetnWord(char *s, int n, char *word, int count)
{
   /* gets the n'th word of s and puts it into word */
	char token[256];
	int i = 0;

	if (count_words (s) < n) { /* not enough words to get... */
		strcpy (word, ""); /* retun void string */
		return 0;
	}
	
   while (n) { /* seek Nth token */
		while (isspace(s[i])) /* while not in a word */
			i++;	
			
      n--;
		
		while (!(isspace(s[i])) && (n != 0)) 
			i++; /* exit the word if it's not the chosen one */
   }

	strncpy (token, s+i, count);
	
   if (token) { /* we must clear any trailing blanks and \n */
		i = 0;
		while (!(isspace(token[i])) && i < count)
			i++;

		token[i] = '\0'; /* close the word */
		strcpy (word, token);
      return(1);
   }
   else
      return(0);
}

/* copy the string after the first blank in WORD, COUNT chars max. */ 
int get_last_word (char *s, char *word, int count)
{
	char *firstblank; /* first blank character occurence in S */
	if (count_words (s) < 2) { /* not enough words */
		strcpy (word, "");
		return 0;
	}

	firstblank = index (s, ' ');
	
	/* jump all spaces between two parts */
	while (isspace (*firstblank))
		firstblank++;

	strncpy (word, firstblank, count);
	if (word[strlen(word)-1] == '\n')
		word[strlen(word)-1] = '\0';

	return 1;
}

/*--------------------------------------------------------------------
 int ExtractLine(char *buffer, char *line);

 Extracts a line (ended on \n or \r) and returns it to the caller.
 It returns also a pointer (offset) to the next line start (i).
--------------------------------------------------------------------*/
int ExtractLine(char *buffer, int maxlength, int i, char *line)
{
   int j=0;

   while( buffer[i] != '\n' && buffer[i] != '\r' )
   {
      line[j] = buffer[i];
        j++; i++;
   }
   line[j] = '\0';

   while( buffer[i] == '\r' || buffer[i] == '\n' )
         i++;

   return(i);
}


/*--------------------------------------------------------------------
 int CountLines(char *buffer);

 Counts the number of lines (ended on \n or \r) and returns it.
--------------------------------------------------------------------*/
int CountLines( char *buffer )
{
   int j=0, i=0;

   while( buffer[i] != '\0' )
   {
      if( buffer[i] == '\n' ) j++;
      i++;
   }
   return(j);
}

/*--- error() standard function -------------------------------------*/
void error(const char *cadena)
{
    printf("%s\n", cadena);
    exit(-1);
}


/*--- Perror() standard function ------------------------------------*/
void Perror(const char *cadena)
{
    perror(cadena);
    exit(-1);
}


/*--- Memory allocation macro ---------------------------------------*/
inline void *Malloc(size_t size)
{
    void *puntero;

    if ( (puntero=(void *)malloc(size)) == NULL)
    {
      perror("Malloc:");
     exit(1);
    }
    return( puntero );
}


/*--------------------------------------------------------------------
 char GetFirstWord( char *buffer, char *word );

 Extracts the first word from a line (ignoring spaces and so on).
--------------------------------------------------------------------*/
char GetFirstWord( char *buffer, int maxsize, char *word )
{
   int i = 0, j = 0;

    while( buffer[i] == ' ' || buffer[i] == '\t' ||
           buffer[i] == '\r' || buffer[i] == '\n' || buffer[i] == '\0' )
    {
        i++;
        if( i > maxsize )   return(0);
    }

   while( buffer[i] != ' ' && buffer[i] != '\t' &&
           buffer[i] != '\r' && buffer[i] != '\n' && buffer[i] != '\0')
   {
        word[j] = buffer[i];
        j++;
        i++;
        if( i > maxsize )  return(0);
   }

   word[j] = '\0';

   return(1);
}


/*--------------------------------------------------------------------
 char GetYesNo( void );

 Waits for YES or NO for english (y/n) and spanish (s/n).
--------------------------------------------------------------------*/
char GetYesNo( void )
{
   char ans;

   fflush(stdin);
   ans = getchar();
   if( ans == 's' || ans == 'S' ||
        ans == 'y' || ans == 'Y' )
           return(1);

   return(0);
}


